package com.loit.common.script;

import com.loit.common.script.dto.BackendFrontEnum;
import com.loit.common.script.dto.DeployInfoDataDTO;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.common.utils.excel.ImportExcel;
import com.loit.common.utils.file.FileUtils;
import com.loit.common.utils.freemarker.FreeMarkerUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.entity.ContentType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class GeneratorScript {


    protected static String root_path = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\bin";


    protected static String root_path_full = root_path + "\\serverTest";


    public static void main(String[] args) {

        try {

            //String filePathStr = Thread.currentThread().getContextClassLoader().getResource("deployInfo.xlsx").getPath();
            String filePathStr = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\deployInfo.xlsx";

            File pdfFile = new File(filePathStr);
            FileInputStream fileInputStream = new FileInputStream(pdfFile);
            MultipartFile multipartFile = new MockMultipartFile(pdfFile.getName(), pdfFile.getName(),

                    ContentType.APPLICATION_OCTET_STREAM.toString(), fileInputStream);
            ImportExcel ei = new ImportExcel(multipartFile, 1, 0);
            List<DeployInfoDataDTO> grayVersionDataDTOList = ei.getDataList(DeployInfoDataDTO.class);

            if (ListUtil.isEmpty(grayVersionDataDTOList)) {
                return;
            }

            for (DeployInfoDataDTO grayVersionDataDTO : grayVersionDataDTOList) {
                buildInitEnv(grayVersionDataDTO);
                buildBackendConfig(grayVersionDataDTO);
                buildFrontConfig(grayVersionDataDTO);
            }

        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }

    }

    /**
     * 生成InitEnv
     *
     * @throws IOException
     */
    private static void buildInitEnv(DeployInfoDataDTO grayVersionDataDTO) throws IOException {

        String fileDir = root_path_full + "\\" + grayVersionDataDTO.getFixedIp();
        File folder = new File(fileDir);
        if (!folder.exists()) {
            folder.mkdirs();
        }

        String nginxConfigDir = root_path_full + "\\" + grayVersionDataDTO.getFixedIp() + "\\nginxConfig";
        File nginxConfigFolder = new File(nginxConfigDir);
        if (!nginxConfigFolder.exists()) {
            nginxConfigFolder.mkdirs();
        }


        String filePath = fileDir + "\\initEnv.sh";
        File file = new File(filePath);
        if (!file.exists()) {
            FileUtils.write(filePath, "#!/bin/bash ");

            //TODO
            FileUtils.appendNewLine(filePath, "rm -rf /home/soft/*");
        }

        String deployPath = grayVersionDataDTO.getDeployPath();
        if (StringUtils.isNoneBlank(deployPath)) {
            FileUtils.appendNewLine(filePath, "mkdir -p " + deployPath);
            BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(grayVersionDataDTO.getBackendFrontType());
            if (BackendFrontEnum.BACKEND.equals(backendFrontType)) {
                FileUtils.appendNewLine(filePath, "mkdir -p " + deployPath + "/logs");
            }
        }


        String nginxConfigShFileName = fileDir + "\\copyNginxConfig.sh";
        String nginxConfigShResult = FreeMarkerUtils.process("copyNginxConfig.ftl", null);
        FileUtils.write(nginxConfigShFileName, nginxConfigShResult);

    }


    /**
     * 生成bootstrap.properties 及其 sh
     */
    private static void buildBackendConfig(DeployInfoDataDTO grayVersionDataDTO) throws IOException {

        String serviceName = grayVersionDataDTO.getServiceName();
        String fixedIp = grayVersionDataDTO.getFixedIp();
        String port = grayVersionDataDTO.getPort();
        String deployPath = grayVersionDataDTO.getDeployPath();
        String deployJar = grayVersionDataDTO.getDeployJar();
        String serviceNameAbb = grayVersionDataDTO.getServiceNameAbb();
        String springProfilesActive = grayVersionDataDTO.getSpringProfilesActive();

        if (StringUtils.isBlank(serviceName)) {
            return;
        }

        BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(grayVersionDataDTO.getBackendFrontType());
        if (!BackendFrontEnum.BACKEND.equals(backendFrontType)) {
            return;
        }

        String folderName = deployPath.replace("/home/soft/", "");
        String fileDir = root_path_full + "\\" + fixedIp + "\\" + folderName;
        File folder = new File(fileDir);
        if (!folder.exists()) {
            folder.mkdirs();
        }

        String bootstrapFileName = fileDir + "\\bootstrap.properties";

        Map model = new HashMap();
        model.put("port", port);
        model.put("serviceName", serviceName);
        model.put("deployPath", deployPath);
        model.put("deployJar", deployJar);
        model.put("serviceNameAbb", serviceNameAbb);
        model.put("springProfilesActive", springProfilesActive);

        String result = FreeMarkerUtils.process("bootstrap.ftl", model);
        FileUtils.write(bootstrapFileName, result);


        String deployShFileName = fileDir + "\\deploy-" + serviceName + "-" + port + ".sh";
        String deployShResult = FreeMarkerUtils.process("deploy-sh.ftl", model);
        FileUtils.write(deployShFileName, deployShResult);
    }


    /**
     * 生成bootstrap.properties 及其 sh
     */
    private static void buildFrontConfig(DeployInfoDataDTO grayVersionDataDTO) throws IOException {

        String serviceName = grayVersionDataDTO.getServiceName();
        String fixedIp = grayVersionDataDTO.getFixedIp();
        String port = grayVersionDataDTO.getPort();
        String deployPath = grayVersionDataDTO.getDeployPath();
        String deployJar = grayVersionDataDTO.getDeployJar();
        String serviceNameAbb = grayVersionDataDTO.getServiceNameAbb();
        String nginxConfigName = grayVersionDataDTO.getNginxConfigName();


        if (StringUtils.isBlank(serviceName)) {
            return;
        }

        BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(grayVersionDataDTO.getBackendFrontType());
        if (BackendFrontEnum.BACKEND.equals(backendFrontType)) {
            return;
        }

        String folderName = deployPath.replace("/home/soft/", "");
        String fileDir = root_path_full + "\\" + fixedIp + "\\" + folderName;
        File folder = new File(fileDir);
        if (!folder.exists()) {
            folder.mkdirs();
        }

        Map model = new HashMap();
        model.put("port", port);
        model.put("serviceName", serviceName);
        model.put("deployPath", deployPath);
        model.put("deployJar", deployJar);
        model.put("serviceNameAbb", serviceNameAbb);
        model.put("nginxConfigName", nginxConfigName);

        String deployShFileName = fileDir + "\\deploy-" + serviceName + "-" + port + ".sh";
        String deployShResult = FreeMarkerUtils.process("deploy-web-sh.ftl", model);
        if ("portal-web".equals(serviceName)) {
            deployShResult = FreeMarkerUtils.process("deploy-portal-web-sh.ftl", model);
        }

        FileUtils.write(deployShFileName, deployShResult);

        if (StringUtils.isNotBlank(nginxConfigName)) {
            String nginxName = root_path_full + "\\" + fixedIp + "\\nginxConfig" + "\\" + nginxConfigName;
            String nginxConfigResult = FreeMarkerUtils.process("nginx_config.ftl", model);
            FileUtils.write(nginxName, nginxConfigResult);
        }

    }
}
